
/* Nelder-Mead simplex search for minimum 
   adapted from
   J. H. Mathews, K. D. Fink, Numerical Methods, pp.441
	T. Sauer, Numerical Analysis, pp.590
*/

fmin = function (func, x) -> xmin

     n = x.length;
     numberOfVariables = n;

     tolx = 1e-4;
     tolf = 1e-4;
     maxfun = 200 * n;
     maxiter = maxfun;

     rho = 1; chi = 2; psi = 0.5; sigma = 0.5;

     xin = x[:];

     v = zeros(n,n+1); fv = zeros(1,n+1);

     v[:,1] = xin;

     x[:] = xin;

     fv[:,1] = func(x);

     usual_delta = 0.05;
     zero_term_delta = 0.00025;

     func_evals = 0;
     itercount=0;

     for j = 1:n
          y = xin;
          if y[j] != 0
             y[j] = (1 + usual_delta)*y[j];
          else 
             y[j] = zero_term_delta;
          end  
          
          v[:,j+1] = y;

          x = y; f = func(x);

          ++func_evals;

          fv[1,j+1] = f;
     end

     j = indexsort(fv);
     fv = fv[j];
     v = v[:,j];


     while func_evals < maxfun && itercount < maxiter
          if max(max(abs(v[:, 2 : n+1]-v[:,1:n]))) <= tolx &&  max(abs(fv[1]-fv[2 : n+1])) <= tolf
             break
          end

          xbar = rowmean(v[:, 1:n]);
          
          xr = (1 + rho)*xbar - rho*v[:,$];
          
          x = xr;
          fxr = func(x);
          
          func_evals = func_evals+1;
          
          if fxr < fv[:,1]
             xe = (1 + rho*chi)*xbar - rho*chi*v[:,$];
             x[:] = xe; fxe = func(x);
             func_evals = func_evals+1;
             if fxe < fxr
                v[:,$] = xe;
                fv[:,$] = fxe;
                how = 'expand';
             else
                v[:,$] = xr; 
                fv[:,$] = fxr;
                how = 'reflect';
             end
          else 
             if fxr < fv[:,n]
                v[:,$] = xr; 
                fv[:,$] = fxr;
                how = 'reflect';
             else 
                if fxr < fv[:,$]
                   xc = (1 + psi*rho)*xbar - psi*rho*v[:,$];
                   x = xc;
                   fxc = func(x);

                   func_evals = func_evals+1;
                   
                   if fxc <= fxr
                      v[:,$] = xc; 
                      fv[:,$] = fxc;
                      how = "contract outside";
                   else
                      how = 'shrink'; 
                   end
                else
                   xcc = (1-psi)*xbar + psi*v[:,$];
                   x = xcc;
                   fxcc = func(x);
                   func_evals = func_evals+1;
                   
                   if fxcc < fv[:,$]
                      v[:,$] = xcc;
                      fv[:,$] = fxcc;
                      how = "contract inside";
                   else
                      how = 'shrink';
                   end
                end
          
                if how == "shrink"
                   for j=2:n+1
                      v[:,j]=v[:,1]+sigma*(v[:,j] - v[:,1]);
                      x = v[:,j];
                      fv[:,j] = func(x);
                   end
                   func_evals = func_evals + n;
                end
             end
          end
          
          j = indexsort(fv);
          fv = fv[j];
          v = v[:,j];
          itercount = itercount + 1;
     end 


     x = v[:,1];

     fval = min(fv); 

     xmin = x;
end
