

/* full rank least square solution to Ax=b
   A is full rank matrix with m>=n

	return xLS: least square solution
	       res: residue

   translated from matlab file (C F. Van Loan, Introduction to scientific
	computing
*/

global.Rotate = function (x1,x2) -> (c,s) 
// [c,s] = Rotate(x1,x2)
// x1 and x2 are real scalars and c and s is a cosine-sine pair so
// -s*x1 + c*x2 = 0.

   if x2==0
   c = 1;
   s = 0;
   else
        if abs(x2)>=abs(x1)
           cotangent = x1/x2;
           s = 1/sqrt(1+cotangent^2);
           c = s*cotangent;
        else
           tangent = x2/x1;
           c = 1/sqrt(1+tangent^2);
           s = c*tangent;
        end
   end
end

global.UTriSol = function (U,b) -> x
   // x = UTriSol(U,b)

   // Solves the nonsingular upper triangular system  Ux = b.
   // where U is n-by-n, b is n-by-1, and X is n-by-1.

   n = length(b);
   x = zeros(n,1);
   for j=n:-1:2
      x[j] = b[j]/U[j,j];
      b[1:j-1] -= x[j]*U[1:j-1,j];
   end
   x[1] = b[1]/U[1,1];
end

global.lsq = function (A, b) -> (xLS, res)

        (m,n) = A.size;

        for j=1:n
             for i=m:-1:j+1
                // Zero A(i,j)
                (c,s) = Rotate(A[i-1,j],A[i,j]);
                A[i-1:i,j:n] = [c, s; -s, c]*A[i-1:i,j:n];
                b[i-1:i] = [c, s; -s, c]*b[i-1:i];
             end
        end

        xLS = UTriSol(A[1:n,1:n],b[1:n]);
        if m==n
          res = 0;
        else
          res = norm(b[n+1:m]);
        end
end


