

// Solve Laplace's equation with Jacob's relaxation method
// Usage: lpc(w,h,bv_up,bv_down,bv_left,bv_right,minchange)
// w: width, h: height, bv_up...bv_down: boundary values, min_change: precision

lpc = function (w, h, bv_up, bv_down, bv_left, bv_right, min_change = 1e-5) -> v

     v=(bv_up+bv_down+bv_left+bv_right)*0.25*ones(w,h); % a crude guess initial value
     vave=zeros(w-2,h-2);
     //boundary conditions
     v[1,:]=bv_left*ones(1,h);
     v[w,:]=bv_right*ones(1,h);
     v[:,1]=bv_down*ones(w,1);
     v[:,h]=bv_up*ones(w,1);

     change=10;

     while change>min_change
        vave=(v[2:w-1,1:h-2]+v[1:w-2,2:h-1]+v[2:w-1,3:h]+v[3:w,2:h-1])*0.25;

        change=max(max(abs(vave-v[2:w-1,2:h-1])));

        v[2:w-1,2:h-1]=vave;
     end
end

