

// spline interpolation
// input: x, y --- the x and y coordinates of the points to be interpolated
// output: P --- the piewise polynomial of natural spline
spline_interp = function (x, y) -> P
    n = x.length;
    h = diff(x);
    d = zeros(n);
    d[1] = 1;
    d[n] = 1;
    d[2 : n-1] = 2 * (h[2 : n-1] + h[1 : n-2]);
    u = zeros(n - 1);
    u[2 : n - 1] = h[2 : n - 1];
    l = h[1 : n-2];
    l[n-1] = 0;
    dy = diff(y);
    m = zeros(n);
    m[2 : n-1] = 3*(dy[2 : n-1] ./ h[2:n-1] - dy[1: n-2] ./ h[1 : n-2]);
    A = band(n, 1, 1);
    A[\] = d;
    A[\, 1] = u;
    A[\, -1] = l;
    c = reshape(A \ m, 1, n)
    // construct the cubic spline
    cc = reshape(c[1 : n-1], 1, n-1);
    aa = reshape(y[1 : n-1], 1, n-1);
    dd = diff(c) ./ (3 * h);
    bb = dy ./ h - (2 * cc + c[2 : n]) .* h / 3;

    P = function [xnodes = x, aa = aa, bb = bb, cc = cc, dd = dd] x -> y
       y = zeros(x.length);
       n = xnodes.length;
       for k = 1 : n - 1
            if k == 1
                 I = sfindall(x, (-inf to xnodes[2]));
            elseif k == n - 1
                 I = sfindall(x, (xnodes[n - 1] to inf));
            else
                 I = sfindall(x, (xnodes[k] to xnodes[k + 1]));
            end
            if I != null
                 xI = x[I];
                 z = xI - xnodes[k];
                 yI = aa[k] + z .* (bb[k] + z .* (cc[k] + z * dd[k]));
                 y[I] = yI;
            end
       end
    end
end

// Note that the function can return a piecewise function, which can be evaluated
// directly. For example

x = [1, 2, 3, 4, 5, 6];
y = [16, 18, 23, 17, 15, 12];
S = spline_interp(x, y);
// check the coefficients
S.aa
xx = linspace(1, 6);
// evaluate spline at xx values
yy = S(xx);

